#!/bin/zsh

# Copyright 2014, 2015 Paul Hänsch
#
# This file is part of Serve0.
# 
# Serve0 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Serve0 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Serve0.  If not, see <http://www.gnu.org/licenses/>. 

export LC_ALL=C

# basic functions
die() {
  echo "$@" >&2
  exit 1
}
debug() { #change to false to disable debugging
  #true && echo "$*" >>debug
  true && [ -n "$*" ] && echo -E "$*" >>/dev/stderr
  true && [ -z "$*" ] && tee /dev/stderr
}

# this program is supposed to be symlinked into a http root directory
# we will use the http root as object storage (data directory) and call sub
# programs from the directory in which the real executable resides
# therefore we need to identify the code and data directories _EXEC and _DATA
call="$0"
real="$(readlink -f $call)"
_EXEC="$(dirname "$real")"  #execution directory
_DATA="$(dirname "$call")"  #storage directory

[ -w "$_DATA" ] && [ -d "$_DATA" ] || die "storage directory must be writable"

# create directories for object storage
for each in "$@"; do
  [ ! -e "$_DATA/$each" ] && mkdir "$_DATA/$each"
  [ -w "$_DATA/$each" ] && [ -d "$_DATA/$each" ] || die "storage \"$_DATA/$each\" must be a writable directory"
done

# create htaccess file
[ -f "$_DATA/.htaccess" ] || cat >"$_DATA/.htaccess" <<EOF
Options		+ExecCGI
AddHandler	cgi-script .cgi
DirectoryIndex	index.cgi
EOF
[ -f "$_DATA/.htaccess" ] || die "no htaccess file present and unable to create one"

. "$_EXEC/shcgi/cgi.sh"

cgi_get

[ -x "$_EXEC/constants.sh" ] && . "$_EXEC/constants.sh"

if [ -n "$_GET[\"action\"]" ]; then
  . "$_EXEC/shcgi/action.sh"
elif [ -n "$_GET[\"export\"]" ]; then
  . "$_EXEC/shcgi/export.sh"
else
  . "$_EXEC/shcgi/page.sh"
fi
